//-*-c++-*------------------------------------------------------------
//
// File name : bioArithDivide.h
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue Apr 28 20:57:24 2009
//
//--------------------------------------------------------------------

#ifndef bioArithDivide_h
#define bioArithDivide_h

class bioArithCompositeLiteral ;
#include "bioArithBinaryExpression.h"

/*!
Class implementing a node for a division operation
*/
class bioArithDivide : public bioArithBinaryExpression {

public:
  
  bioArithDivide(bioExpressionRepository* rep,
		 patULong par,
		 patULong left, 
		 patULong right,
		 patError*& err) ;
  ~bioArithDivide() ;

public:
  virtual patString getOperatorName() const;
  virtual patReal getValue(patBoolean prepareGradient, patError*& err)  ;
  virtual bioExpression* getDerivative(patULong aLiteralId,
				       patError*& err) const ;
  virtual bioArithDivide* getDeepCopy(bioExpressionRepository* rep,
				      patError*& err) const ;
  virtual bioArithDivide* getShallowCopy(bioExpressionRepository* rep,
				      patError*& err) const ;
  virtual patString getCppCode(patString prefix, patError*& err) ;
  virtual patString getSimpleCppCode(patError*& err) ;
  // The simpleCppCode does not work because we need to check if
  // numerator is zero before computing the ratio. A segmentation
  // fault is triggered. Did not have time to investigate more, so I
  // prefer to declare this expression has "not simple".
  virtual patBoolean isSimple() const ;
  virtual patBoolean isStructurallyZero() const ;
  virtual patString getExpressionString() const ;
  virtual bioFunctionAndDerivatives* getNumericalFunctionAndGradient(vector<patULong> literalIds, patError*& err) ;

private:
  bioArithCompositeLiteral* theLeftLiteral ;
  patULong theLeftLiteralCompId ;
  bioArithCompositeLiteral* theRightLiteral ;
  patULong theRightLiteralCompId ;
  bioArithCompositeLiteral* theRightSquaredLiteral ;
  patULong theRightSquaredLiteralCompId ;


};

#endif
