//-*-c++-*------------------------------------------------------------
//
// File name : bioArithNamedExpression.h
// Author :    Michel Bierlaire
// Date :      Tue Apr 12 13:13:47 2011
//
//--------------------------------------------------------------------

#ifndef bioArithNamedExpression_h
#define bioArithNamedExpression_h


#include "bioArithBinaryExpression.h"

class bioCompositeLiteral ;

/*!
Class implementing a named expression, that is the association of an expression with the name of literal. The expression is of the form
  literal = expression
where literal is the left child, and expression is the right child. 
The value of the expression is the value of its right child.
*/
class bioArithNamedExpression : public bioArithBinaryExpression {

 public: 
  bioArithNamedExpression(bioExpressionRepository* rep, 
			  patULong par,
			  patString literalName,
			  patULong expr, 
			  patError*& err) ;

  bioArithNamedExpression(bioExpressionRepository* rep,
			  patULong par,
			  patULong compositeId,
			  patULong expr, 
			  patError*& err) ;


  ~bioArithNamedExpression() ;
  virtual patString getOperatorName() const;
  virtual patReal getValue(patBoolean prepareGradient, patError*& err)  ;
  virtual bioExpression* getDerivative(patULong aLiteralId, 
				       patError*& err) const ;
  virtual bioArithNamedExpression* getDeepCopy(bioExpressionRepository* rep,
					       patError*& err) const ;
  virtual bioArithNamedExpression* getShallowCopy(bioExpressionRepository* rep,
					       patError*& err) const ;
  virtual patString getSimpleCppCode(patError*& err) ;
  virtual patString getCppCode(patString prefix, patError*& err) ;
  virtual patString getExpressionString() const ;

  const bioCompositeLiteral* getLiteral() const ;

  patBoolean isNamedExpression() const ;
  patBoolean isStructurallyZero() const ;

  // A named expression is trivial when it is NAME = NAME
  // It happens when it is created by the function createNamedExpression
  patBoolean isTrivial() const ;

  bioExpression* getExpressionOfTheLiteral() const ;

  virtual bioFunctionAndDerivatives* getNumericalFunctionAndGradient(vector<patULong> literalIds, patError*& err) ;
 private:
  patULong theLiteralCompId ;

};

#endif
