//-*-c++-*------------------------------------------------------------
//
// File name : bioArithRandom.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Thu Jul 30 17:19:12 2009
//
//--------------------------------------------------------------------

#include "bioArithRandom.h"
#include "bioArithConstant.h"
#include "patDisplay.h"
#include "patErrMiscError.h"
#include "patErrNullPointer.h"
#include "bioRandomDraws.h"
#include "bioLiteralRepository.h"
#include "bioExpressionRepository.h"

bioArithRandom::bioArithRandom(bioExpressionRepository* rep,
			       patULong par, 
			       patULong individual,
			       patULong i, 
			       patString t,
			       patError*& err) :
  bioArithUnaryExpression(rep, par, individual,err),
  variableIdForDraws(i),
  type(t) {

}
bioArithRandom::~bioArithRandom() {
} ;

patString bioArithRandom::getOperatorName() const {
  stringstream str ;
  str << type << "(" << variableIdForDraws << ")" ;
  return patString(str.str()) ;
}

patReal bioArithRandom::getValue(patBoolean prepareGradient, patError*& err)  {
#ifdef BIO_CHECK
  if (__draws == NULL) {
    err = new patErrNullPointer("__draws") ;
    WARNING(err->describe()) ;
    return patReal() ;
  }
#endif
  patReal c = child->getValue(prepareGradient,err) ;
#ifdef BIO_CHECK
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal() ;
  }
#endif
  return (*__draws)[variableIdForDraws][patULong(c)] ;

}

bioExpression* bioArithRandom::getDerivative(patULong aLiteralId, patError*& err) const {
  bioExpression* result = theRepository->getZero() ;
  return result ;
}

bioArithRandom* bioArithRandom::getDeepCopy(bioExpressionRepository* rep,
					    patError*& err) const {
  bioExpression* clone = child->getDeepCopy(rep,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  bioArithRandom* theNewNode = new 
    bioArithRandom(rep,patBadId,clone->getId(),variableIdForDraws,type,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  return theNewNode ;

}

bioArithRandom* bioArithRandom::getShallowCopy(bioExpressionRepository* rep,
					       patError*& err) const {
  bioArithRandom* theNewNode = new 
    bioArithRandom(rep,patBadId,child->getId(),variableIdForDraws,type,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  return theNewNode ;

}

patString bioArithRandom::getSimpleCppCode(patError*& err) {
  stringstream str ;
  str << "(*__draw)[ " << variableIdForDraws << "][patULong(" <<child->getSimpleCppCode(err) << ")]" ;
  return patString(str.str()) ;
    
}

patString bioArithRandom::getCppCode(patString prefix, patError*& err) {
  if (codeMustBeGenerated) {
  stringstream str  ;
  str << getCppResultDeclaration(prefix) << "=" <<  getSimpleCppCode(err) << " ;" << endl ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString() ;
  }
  //  str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ");" << endl ;
  codeMustBeGenerated = patFALSE ;
  return patString(str.str()) ;
  }
  else {
    return patString() ;
  }
}

patULong bioArithRandom::getNumberOfOperations() const {
  return 0 ;
}

patString bioArithRandom::getExpressionString() const {
  stringstream str ;
  str << "&" << variableIdForDraws ;
  return patString(str.str()) ;

}

patString bioArithRandom::getExpression(patError*& err) const {
  return getOperatorName() ;
}

patBoolean bioArithRandom::dependsOf(patULong aLiteralId) const {
  return patFALSE ;
}


bioFunctionAndDerivatives* bioArithRandom::getNumericalFunctionAndGradient(vector<patULong> literalIds, patError*& err) {
  if (result.empty()) {
    result.resize(literalIds.size(),0.0) ;
  }
  result.theFunction = getValue(patFALSE,err) ;
#ifdef BIO_CHECK
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
#endif
  return &result ;
  
}
